// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: rollback.cc 97 2006-11-27 19:52:44Z jpra $
/* ######################################################################

   Rollback - TODO: Summary
 
   TODO: Description

   AUTHOR: David Salvador Pinheiro <david.pinheiro@caixamagica.pt>

   ##################################################################### */
									/*}}}*/
// Include Files							/*{{{*/
#ifdef __GNUG__
#pragma implementation "apt-pkg/rollback.h"
#endif
#include <apt-pkg/rollback.h>
#include <apt-pkg/error.h>
#include <apt-pkg/configuration.h>
#include <apt-pkg/sptr.h>
#include <apt-pkg/pkgsystem.h>
#include <apt-pkg/version.h>
#include <apt-pkg/versionmatch.h>
#include <apt-pkg/algorithms.h>
#include <apt-pkg/fileutl.h>
#include <apt-pkg/tar.h>

#include <apti18n.h>
#include <config.h>

#include <iostream>
#include <sstream>
#include <string>
#include <list>

#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>

#ifdef WITH_SQLITE3
#include <sqlite3.h>
#else

#define SQLITE_OK 0

static int sqlite3_open(const char *, sqlite3 **)
{
    return SQLITE_OK;
}

static int sqlite3_close(sqlite3 *)
{
    return SQLITE_OK;
}

static int sqlite3_exec(sqlite3 *, const char *, void *, void *, void *)
{
    return SQLITE_OK;
}

static int sqlite3_get_table(sqlite3 *, const char *, char ***, int * nRow, int * nCol, char **)
{
    if (nRow)
        *nRow = 0;

    if (nCol)
        *nCol = 0;

    return SQLITE_OK;
}

static void sqlite3_free_table(char **)
{
    return;
}

static const char * sqlite3_errmsg(sqlite3 *)
{
    return "this version does not have sqlite support compiled in";
}

#endif
									/*}}}*/

#define PKG_HISTORY_TABLE   "pkgHistory"

using namespace std;

unsigned int GetSystemTime()
{
   struct timeval tv;
   gettimeofday(&tv, NULL);

   return (unsigned int) tv.tv_sec;
}

//AG: This construtor skips pkglist reloading, to be used by rollback-hist
pkgRollbackManager::pkgRollbackManager(): CfgBck(0), hDb(0)
{
   Debug = _config->FindB("Debug::pkgRollbackManager", false);

   if (OpenDB() == false)
      _error->Error(_("Unable to open Rollback Database"));

   // Check whether the SQL tables exist
   if (DBTablesExist() == false)
   {
      // SQL Tables don't exist, lets create them
      if (CreateDBTables() == false)
      {
	 //Ok, there was an error creating the SQL tables, this shouldn't happen.
	 CloseDB();
      }
   }

   TIDCache = -1;
}

pkgRollbackManager::pkgRollbackManager(pkgDepCache *Cache) : Cache(Cache), CfgBck(0), hDb(0)
{
   Debug = _config->FindB("Debug::pkgRollbackManager", false);

   if (OpenDB() == false)
      _error->Error(_("Unable to open Rollback Database"));

   // Check whether the SQL tables exist
   if (DBTablesExist() == false)
   {
      // SQL Tables don't exist, lets create them
      if (CreateDBTables() == false)
      {
	 //Ok, there was an error creating the SQL tables, this shouldn't happen.
	 CloseDB();
      }
   }

   TIDCache = -1;
}

pkgRollbackManager::~pkgRollbackManager()
{
   CloseDB();
}


bool pkgRollbackManager::LogPkgInstall(PkgIterator Pkg, string File)
{
   if (Debug)
      cout << "pkgRollbackManager::LogPkgInstall ( " << Pkg.Name() << " )"<< endl;

   //PkgIterator Pkg = iPkg;//Cache->FindPkg(iPkg.Name()); 
   VerIterator CandVer = (*Cache)[Pkg].CandidateVerIter(*Cache);
   VerIterator CurrVer = Pkg.CurrentVer();
   string VerStr1, VerStr2;
   
   
   if (File.empty() == true || Pkg.end() == true)
      return _error->Error(_("Internal Error, No file name for %s"), Pkg.Name());
   
   Item::Ops Op = Item::Install;
   if ((*Cache)[Pkg].NewInstall() == true)
   {
      Op = Item::Install;
      //VerStr1 = (*Cache)[Pkg].CandVersion;
      VerStr1 = CandVer.VerStr();
      VerStr2 = "";
   }
   else if ((*Cache)[Pkg].Upgrade()== true)
   {
      Op = Item::Upgrade;
      VerStr1 = CurrVer.VerStr();
      VerStr2 = CandVer.VerStr();
   }
   else if ((*Cache)[Pkg].Downgrade() == true)
   {
      Op = Item::Downgrade;
      VerStr1 = CurrVer.VerStr();
      VerStr2 = CandVer.VerStr();
   }
   else if ((*Cache)[Pkg].Delete() == true)
   {
      Op = Item::Remove;
      VerStr1 = CurrVer.VerStr();
      VerStr2 = "";
   }
   /*if ((*Cache)[Pkg].Upgrade() == true)
      Op = Item::Upgrade;
   else if ((*Cache)[Pkg].Downgrade() == true)
      Op = Item::Downgrade;
   else if ((*Cache)[Pkg].Install() == true)
      Op = Item::Install;
*/
   
   List.push_back(Item(Op, Pkg.Name(), VerStr1, VerStr2, GetSystemTime(), File));
   return true;
}

bool pkgRollbackManager::LogPkgRemove(PkgIterator Pkg, bool Purge)
{
   VerIterator Ver;
   string VerStr1, VerStr2 = "";
   Ver = Pkg.CurrentVer();

   if (Ver.end() == true)
      return false;
   if (Pkg.end() == true)
      return false;
   VerStr1 = Ver.VerStr();
   if (Purge == true)
      List.push_back(Item(Item::Purge, Pkg.Name(), VerStr1, VerStr2, GetSystemTime()));
   else
      List.push_back(Item(Item::Remove, Pkg.Name(), VerStr1, VerStr2, GetSystemTime()));
   return true;
}

bool pkgRollbackManager::Commit()
{
   //Generate or get last transaction ID
   int TransID = GenerateTransactionID();
   for (vector<Item>::iterator I = List.begin(); I != List.end(); I++)
   {
      char Op = 'N';
      switch (I->Op)
      {
	 case Item::Purge:
	 case Item::Remove:
	 Op = 'R';
	 break;

	 case Item::Install:
	 Op = 'I';
	 break;

	 case Item::Upgrade:
	 Op = 'U';
	 break;

	 case Item::Downgrade:
	 Op = 'D';
	 break;
      }

      InsertDB(I->Pkg, I->Ver1, I->Ver2, TransID, Op, I->TimeStamp);
   }

   return true;
}

bool pkgRollbackManager::InsertDB(const string &pkgName, const string &pkgVer1, const string &pkgVer2, int Parent, const char Op, unsigned int TimeStamp)
{
   if (Debug == true)
      cout << "pkgRollbackManager::InsertDB(" << pkgName << "," << pkgVer1 << "," << Parent << "," << Op << "," << TimeStamp << ")" << endl;

   int Rc;
   ostringstream SQL;

   SQL << "INSERT INTO " << PKG_HISTORY_TABLE << " VALUES( NULL,";
   SQL << Parent << ",";
   SQL << "'" << Op << "',";
   SQL << "'" << pkgName << "',";
   SQL << "'" << pkgVer1 << "',";
   SQL << "'" << pkgVer2 << "',";
   SQL << TimeStamp << ")";

   if (_config->FindB("APT::Get::Simulate") == true)
   {
      cout << SQL.str() << endl;
      return true;
   }

   Rc = sqlite3_exec(hDb, SQL.str().c_str(), NULL, NULL, NULL);

   if (Rc != SQLITE_OK)
   {
      string err(sqlite3_errmsg(hDb));
      cerr << SQL.str() << endl;
      _error->Error(_("sqlite error: %s"),err.c_str());

      return false;
   }

   return true;
}

bool pkgRollbackManager::OpenDB()
{
   int Rc = 0;

   string FName = _config->FindFile("Dir::State::rollback");

   Rc = sqlite3_open(FName.c_str(), &hDb);
   if (Rc)
   {
      string err(sqlite3_errmsg(hDb));
      _error->Error(_("sqlite error: %s"),err.c_str());
      sqlite3_close(hDb);
      hDb = 0;

      return false;
   }

   return true;
}

void pkgRollbackManager::CloseDB()
{
   if (hDb)
   {
      sqlite3_close(hDb);
      hDb = 0;
   }
}

int pkgRollbackManager::GenerateTransactionID()
{
   int Rc;
   char **Result;
   ostringstream SQL;
   int nRow;
   int nColumn;
   int ID;

   SQL << "SELECT IFNULL(MAX(parent), 0) FROM " <<  PKG_HISTORY_TABLE << endl;

   Rc = sqlite3_get_table(hDb, SQL.str().c_str(), &Result, &nRow, &nColumn, NULL);

   if (Rc != SQLITE_OK || nRow != 1 || nColumn == 0)
   {
      //handle the error;
      if (Debug)
	 cout << "pkgRollbackManager::GetTransactionID()" <<endl;
      _error->Error(_("Unexpected error in getting the transaction ID"));

      ID = -1;
   }
   else if (Result[nColumn] == NULL)
      ID = 1;
   else
      ID = (atoi(Result[nColumn])+1);

   sqlite3_free_table(Result);
   TIDCache = ID;

   return ID;
}

int pkgRollbackManager::GetTID()
{
   if (TIDCache == -1)
      GenerateTransactionID();

   return TIDCache; 
}

bool pkgRollbackManager::CreateDBTables()
{
   int Rc;
   ostringstream SQL;

   SQL << "CREATE TABLE " << PKG_HISTORY_TABLE <<" (" << endl;
   SQL << "   id INTEGER PRIMARY KEY AUTOINCREMENT,"  << endl;
   SQL << "   parent     INTEGER,"		      << endl;
   SQL << "   op         CHAR,"			      << endl;
   SQL << "   pkgName    VARCHAR(300),"		      << endl;
   SQL << "   pkgVer1    VARCHAR(300),"		      << endl;
   SQL << "   pkgVer2    VARCHAR(300),"		      << endl;
   SQL << "   datetime   TIMESTAMP"		      << endl;
   SQL << ");"					      << endl;

   Rc = sqlite3_exec(hDb, SQL.str().c_str(), NULL, NULL, NULL);
   if (Rc != SQLITE_OK)
   {
      cerr << "SQL:" << SQL.str() << endl;
      cerr << sqlite3_errmsg(hDb) << endl;
      return false;
   }

   return true;
}

bool pkgRollbackManager::DBTablesExist()
{
   int Rc;
   ostringstream SQL;
   char **Result;
   int nRow = 0;
   int nColumn = 0;

   SQL << "SELECT name FROM sqlite_master WHERE type='table' AND name='"<<PKG_HISTORY_TABLE<<"'";

   Rc = sqlite3_get_table(hDb, SQL.str().c_str(), &Result, &nRow, &nColumn, NULL);
   if (Rc != SQLITE_OK)
   {
      sqlite3_free_table(Result);
      return false;
   }

   if (nRow == 1)
   {
      sqlite3_free_table(Result);
      return true;
   }

   return false;
}

bool pkgRollbackManager::GetRollbackList(vector<vector<Item> > &List)
{
   list<int> TIDList;

   GetRollbackTID(TIDList);

   if (!List.empty())
      List.clear();

   for (list<int>::iterator I = TIDList.begin(); I != TIDList.end(); I++)
   {
      vector<Item> ItemList;
      GetRollbackList(*I, ItemList);
      List.push_back(ItemList);
   }

   if (List.empty())
      return false;

   return true;
}

bool pkgRollbackManager::GetRollbackList(int TID, vector<Item> &List)
{
   int Rc;
   char **Result;
   ostringstream SQL;
   int nRow;
   int nColumn;

   SQL <<  "SELECT pkgName, pkgVer1, id, parent, op, datetime, pkgVer2 FROM " << PKG_HISTORY_TABLE << " WHERE parent=" << TID << endl;

   Rc = sqlite3_get_table(hDb, SQL.str().c_str(), &Result, &nRow, &nColumn, NULL);
   if (Rc != SQLITE_OK || nColumn < 7)
   {
      //handle the error;
      sqlite3_free_table( Result );
      return false;
   }

   if (List.size() > 0)
      List.clear();

   for (int j = nColumn; j < (nRow + 1) * nColumn; j += nColumn)
   {
      string pkgName = Result[j + 0];
      string pkgVersion1 = Result[j + 1];
      char op = *Result[j + 4];
      unsigned int datetime = (unsigned int) atol(Result[j + 5]);
      string pkgVersion2 = Result[j + 6];
      Item::Ops Op;
      switch(op)
      {
	 case 'i':
	 case 'I':
	 Op = Item::Install;
	 break;

	 case 'R':
	 case 'r':
	 Op = Item::Remove;
	 break;

	 case 'U':
	 case 'u':
	 Op = Item::Upgrade;
	 break;

	 case 'D':
	 case 'd':
	 Op = Item::Downgrade;
	 break;
      }

      List.push_back(Item(Op, pkgName, pkgVersion1, pkgVersion2, datetime));
   }

   sqlite3_free_table( Result );

   return true;
}

bool pkgRollbackManager::GetRollbackTID(list<int> &tidList)
{
   int Rc;
   char **Result;
   ostringstream SQL;
   int nRow;
   int nColumn;

   if (tidList.empty() == false)
      tidList.clear();

   SQL <<  "SELECT distinct(parent) FROM " << PKG_HISTORY_TABLE;

   Rc = sqlite3_get_table(hDb, SQL.str().c_str(), &Result, &nRow, &nColumn, NULL);
   if (Rc != SQLITE_OK)
   {
      //handle the error;
      sqlite3_free_table( Result );
      return false;
   }

   for (int i = nColumn; i < (nRow + 1) * nColumn; i += nColumn)
   {
      string TID = Result[i + 0];
      tidList.push_back(atoi(TID.c_str()));
   }

   sqlite3_free_table( Result );
   return true;
}

/*!
   \fn pkgRollbackManager::Rollback(int TID)
 */
bool pkgRollbackManager::Rollback(int TID)
{
   vector<Item> List;
   
   pkgProblemResolver Fix(Cache);
   
   GetRollbackList(TID, List);
   for (vector<Item>::iterator I = List.begin(); I != List.end(); I++)
   {
      bool Remove = false;
      pkgCache::PkgIterator Pkg = Cache->FindPkg(I->Pkg);
      
      if (I->Op == Item::Purge || I->Op == Item::Remove)
      Remove = false;

      if (I->Op == Item::Install)
      Remove = true;

      if (Pkg.end() == true)
      {
	 // Package is no longer available
	 _error->Error(_("Package %s was not found"),I->Pkg.c_str());
	 return false;
      }
      pkgVersionMatch Match(I->Ver1, pkgVersionMatch::Version, pkgCache::Dep::Equals);
      pkgCache::VerIterator Ver = Match.Find(Pkg);
      if (Ver.end() == true)
      {
	 // Error, selected version was not found
	  _error->Error(_("Package %s with version %s was not found"), I->Pkg.c_str(), I->Ver1.c_str()) ;
	 continue;
      }
      Cache->SetCandidateVersion(Ver);
      Pkg = Ver.ParentPkg();
      pkgDepCache::StateCache &State = (*Cache)[Pkg];
      if (State.CandidateVer == 0 && Remove == false)
      {
	 // TODO: Handle the appropriate error
	 return false;
      }
      Fix.Clear(Pkg);
      Fix.Protect(Pkg);
      if (Remove == true)
      {
	 Fix.Remove(Pkg);
	 Cache->MarkDelete(Pkg, false);
	 continue;
      }
      Cache->MarkInstall(Pkg, false);
      CfgBck->MarkRestore(I->Pkg.c_str(), I->Ver1.c_str(), TID);
      if (State.Install() == true)
      {
	 //package is already installed
	 cout << "Package " << I->Pkg <<"( " << I->Ver1 << " )" << " is already installed" << endl;
	 continue;
      }
   }
   return true;
}

bool pkgRollbackManager::RestoreCfgBackup()
{
   return CfgBck->Restore();
}

// Class  pkgRollbackManager::pkgCfgBackup implementation
string pkgRollbackManager::pkgCfgBackup::FormatFN(const char *pkgName, int TID, const char *pkgVersion, const char *pkgRelease)
{   
   ostringstream FN;
   FN << pkgName <<"-cfg" << TID << "-"<< pkgVersion << "-" << pkgRelease <<".tar.bz2";
   return FN.str();
}

string pkgRollbackManager::pkgCfgBackup::FormatFN(const char *pkgName, int TID, const char *pkgVersion)
{
   ostringstream FN;
   FN << pkgName <<"-cfg" << TID << "-"<< pkgVersion <<".tar.bz2";
   return FN.str();
}

bool pkgRollbackManager::pkgCfgBackup::Create(const char *pkgName, int TID)
{
   vector<string> FileList;
   string FileName;   
   string pkgVersion;
   string pkgRelease;
   
   if (GetCfgFileList(pkgName, FileList, pkgVersion, pkgRelease) == false)
      return false;

   if (FileList.empty())
      return false;

   string Dir = _config->FindDir("Dir::State::pkgcfg");
   if (FileExists(Dir) == false)
   {
      if ( mkdir(Dir.c_str(), 0644) == -1)
	 return false;
   }
   FileName = flCombine(Dir, FormatFN(pkgName, TID, pkgVersion.c_str(), pkgRelease.c_str()));
   BckFileList.push_back(FileName);
   return Tar::Create(FileName.c_str(), FileList, Tar::BZ2, NULL, NULL);
}

bool pkgRollbackManager::pkgCfgBackup::Restore()
{
   bool Ret;
   string CWD = SafeGetCWD();
   string Dir = _config->FindDir("Dir::State::pkgcfg");
   if(chdir ("/") == -1)
        return false;
   
   for (list<string>::iterator I = PkgCfgRestoreList.begin(); I != PkgCfgRestoreList.end(); I++)
   {
      string TarFile = flCombine(Dir, *I);
      if (FileExists(TarFile) == false)
      {
	 //Ret = false;
	 //No configuration files available for this package
	 cout << "Extract error: " << *I << endl;
	 continue;
	 //TODO:LOG ERROR
      }
      Ret = Tar::Extract(TarFile.c_str(), Tar::BZ2, NULL, NULL);
      cout << "Extracted " << TarFile.c_str() << endl;
   }
   if(chdir(CWD.c_str()) == -1)
    return false;   
   else return true;
}

bool pkgRollbackManager::pkgCfgBackup::DeleteBckUPFiles()
{
   bool Ret = true;

   for (list<string>::iterator I = BckFileList.begin(); I != BckFileList.end(); I++)
   {
      if (unlink((*I).c_str()))
      Ret = false;
   }

   return Ret;
}

bool pkgRollbackManager::pkgCfgBackup::MarkRestore(const char *PkgName, const char *Ver, int TID)
{
   //RstBckFiles.push_back
   PkgCfgRestoreList.push_back(FormatFN(PkgName, TID, Ver));
   return true;
}

// vim:sts=3:sw=3
